import { defineNuxtModule, installModule, addComponent, addImportsSources } from '@nuxt/kit';
import * as storefrontUi from '@storefront-ui/vue';
import { defu } from 'defu';
import { tailwindConfig } from '@storefront-ui/vue/tailwind-config';
import '@nuxtjs/tailwindcss';

const module = defineNuxtModule({
  meta: {
    name: "storefront-ui",
    configKey: "storefrontUi",
    compatibility: {
      nuxt: ">=3.0.0"
    }
  },
  // Default configuration options of the Nuxt module
  defaults: {
    contentPath: "./node_modules/@storefront-ui/vue/**/*.mjs"
  },
  async setup(options, nuxt) {
    const { contentPath } = options;
    const nuxtOptions = nuxt.options;
    const defaultTailwindConfig = {
      presets: [tailwindConfig],
      content: [contentPath ?? ""]
    };
    const nuxtConfigTailwindConfig = structuredClone(nuxtOptions.tailwindcss?.config);
    nuxt.hook("tailwindcss:config", function(tailwindFileConfig) {
      const mergeDefaultWithNuxtConfig = defu(nuxtConfigTailwindConfig, defaultTailwindConfig);
      Object.assign(tailwindFileConfig, defu(tailwindFileConfig, mergeDefaultWithNuxtConfig));
    });
    nuxtOptions.tailwindcss = {
      config: {
        plugins: [() => {
        }]
        // DO NOT REMOVE/HACK - this property need to exists otherwise whole plugin won't work. Reason is that if there is no plugin set by default, tailwind on nuxt create template inside `.nuxt` directory that is reused later. Tailwind cant do it, because then it stringify plugins that contains functions to `plugins: [null, null]` which means any plugin we load thorugh our preset won't work. https://github.com/nuxt-modules/tailwindcss/blob/fb8fafb98cbf3ea824a1645156d6484ff6c5eda1/src/context.ts#L47. There is switch if there is no `plugins` property on initial load, config is generated, so any plugin cannot be added.
        // TODO: on 6.12.1 should be fixed https://github.com/nuxt-modules/tailwindcss/blob/main/src/context.ts#L87
      }
    };
    await installModule("@nuxtjs/tailwindcss");
    const components = [];
    const composables = [];
    Object.keys(storefrontUi).forEach((key) => {
      if (key.startsWith("Sf") && (storefrontUi[key].__name || storefrontUi[key].name)) {
        components.push(key);
      } else if (key.startsWith("use")) {
        if (key !== "useId")
          composables.push(key);
      }
    });
    components.forEach((key) => {
      addComponent({
        name: key,
        // name of the component to be used in vue templates,
        export: key,
        filePath: `@storefront-ui/vue`
      });
    });
    addImportsSources({
      imports: composables,
      from: "@storefront-ui/vue"
    });
  }
});

export { module as default };
